@@ -3,6 +3,9 @@ from qiskit.providers.providerutils import filter_backends
 
 from .qasmsimulator import QasmSimulator
 from .statevectorsimulator import StatevectorSimulator
+from .hybridqasmsimulator import HybridQasmSimulator
+from .hybridstatevectorsimulator import HybridStatevectorSimulator
+from .unitarysimulator import UnitarySimulator
 
 
 class JKQProvider(Provider):
@@ -13,6 +16,9 @@ class JKQProvider(Provider):
             JKQProvider._BACKENDS = [
                 ('qasm_simulator', QasmSimulator, None, None),
                 ('statevector_simulator', StatevectorSimulator, None, None),
+                ('hybrid_qasm_simulator', HybridQasmSimulator, None, None),
+                ('hybrid_statevector_simulator', HybridStatevectorSimulator, None, None),
+                ('unitary_simulator', UnitarySimulator, None, None)
             ]
 
     def get_backend(self, name=None, **kwargs):
