@@ -50,6 +50,15 @@ void StochasticNoiseSimulator::perfect_simulation_run() {
                         assert(result == '0' || result == '1');
                         classic_values[classic.at(i)] = (result == '1');
                     }
+                } else if (op->getType() == qc::Reset) {
+                    // Reset qubit
+                    auto quantum = nu_op->getTargets();
+                    for (signed char i: quantum) {
+                        auto result = MeasureOneCollapsing(i, true);
+                        if (result == '1') {
+                            setMeasuredQubitToZero(quantum.at(i), root_edge, dd);
+                        }
+                    }
                 } else if (op->getType() == qc::Barrier) {
                     continue;
                 } else {
@@ -160,162 +169,36 @@ std::map<std::string, double> StochasticNoiseSimulator::StochSimulate() {
     return measure_result;
 }
 
-dd::Package::vEdge StochasticNoiseSimulator::RemoveNodesInPackage(std::unique_ptr<dd::Package>& localDD, dd::Package::vEdge e, std::map<dd::Package::vNode*, dd::Package::vEdge>& dag_edges) {
-    if (e.isTerminal()) {
-        return e;
-    }
-
-    const auto it = dag_edges.find(e.p);
-    if (it != dag_edges.end()) {
-        dd::Package::vEdge r = it->second;
-        if (r.w.approximatelyZero()) {
-            return dd::Package::vEdge::zero;
-        }
-        dd::Complex c = localDD->cn.getTemporary();
-        dd::ComplexNumbers::mul(c, e.w, r.w);
-        r.w = localDD->cn.lookup(c);
-        return r;
-    }
-
-    std::array<dd::Package::vEdge, dd::RADIX> edges{
-            RemoveNodesInPackage(localDD, e.p->e.at(0), dag_edges),
-            RemoveNodesInPackage(localDD, e.p->e.at(1), dag_edges)};
-
-    dd::Package::vEdge r = localDD->makeDDNode(e.p->v, edges, false);
-    dag_edges[e.p]       = r;
-    dd::Complex c        = localDD->cn.getCached();
-    dd::ComplexNumbers::mul(c, e.w, r.w);
-    r.w = localDD->cn.lookup(c);
-    return r;
-}
-
-double StochasticNoiseSimulator::ApproximateEdgeByFidelity(std::unique_ptr<dd::Package>& localDD, dd::Package::vEdge& edge, double targetFidelity, bool allLevels, bool removeNodes) {
-    std::queue<dd::Package::vNode*>       q;
-    std::map<dd::Package::vNode*, dd::fp> probsMone;
-
-    probsMone[edge.p] = dd::ComplexNumbers::mag2(edge.w);
-
-    q.push(edge.p);
-
-    while (!q.empty()) {
-        dd::Package::vNode* ptr = q.front();
-        q.pop();
-        const dd::fp parent_prob = probsMone[ptr];
-
-        if (ptr->e.at(0).w != dd::Complex::zero) {
-            if (probsMone.find(ptr->e.at(0).p) == probsMone.end()) {
-                q.push(ptr->e.at(0).p);
-                probsMone[ptr->e.at(0).p] = 0;
-            }
-            probsMone[ptr->e.at(0).p] = probsMone.at(ptr->e.at(0).p) + parent_prob * dd::ComplexNumbers::mag2(ptr->e.at(0).w);
-        }
-
-        if (ptr->e.at(1).w != dd::Complex::zero) {
-            if (probsMone.find(ptr->e.at(1).p) == probsMone.end()) {
-                q.push(ptr->e.at(1).p);
-                probsMone[ptr->e.at(1).p] = 0;
-            }
-            probsMone[ptr->e.at(1).p] = probsMone.at(ptr->e.at(1).p) + parent_prob * dd::ComplexNumbers::mag2(ptr->e.at(1).w);
-        }
-    }
-
-    std::vector<int> nodes(getNumberOfQubits(), 0);
-
-    std::vector<std::priority_queue<std::pair<double, dd::Package::vNode*>, std::vector<std::pair<double, dd::Package::vNode*>>>> qq(
-            getNumberOfQubits());
-
-    for (auto& it: probsMone) {
-        if (it.first->v < 0) {
-            continue; // ignore the terminal node which has v == -1
-        }
-        nodes.at(it.first->v)++;
-        qq.at(it.first->v).emplace(1 - it.second, it.first);
-    }
-
-    probsMone.clear();
-    std::vector<dd::Package::vNode*> nodes_to_remove;
-
-    int max_remove = 0;
-    for (int i = 0; i < getNumberOfQubits(); i++) {
-        double                           sum    = 0.0;
-        int                              remove = 0;
-        std::vector<dd::Package::vNode*> tmp;
-
-        while (!qq.at(i).empty()) {
-            auto p = qq.at(i).top();
-            qq.at(i).pop();
-            sum += 1 - p.first;
-            if (sum < 1 - targetFidelity) {
-                remove++;
-                if (allLevels) {
-                    nodes_to_remove.push_back(p.second);
-                } else {
-                    tmp.push_back(p.second);
-                }
-            } else {
-                break;
-            }
-        }
-        if (!allLevels && remove * i > max_remove) {
-            max_remove      = remove * i;
-            nodes_to_remove = tmp;
-        }
-    }
-
-    std::map<dd::Package::vNode*, dd::Package::vEdge> dag_edges;
-    for (auto& it: nodes_to_remove) {
-        dag_edges[it] = dd::Package::vEdge::zero;
-    }
-
-    dd::Package::vEdge newEdge = RemoveNodesInPackage(localDD, edge, dag_edges);
-    assert(!std::isnan(dd::CTEntry::val(edge.w.r)));
-    assert(!std::isnan(dd::CTEntry::val(edge.w.i)));
-    dd::Complex c = localDD->cn.getCached(std::sqrt(dd::ComplexNumbers::mag2(newEdge.w)), 0);
-    dd::ComplexNumbers::div(c, newEdge.w, c);
-    newEdge.w = localDD->cn.lookup(c);
-
-    dd::fp fidelity = 0;
-    if (edge.p->v == newEdge.p->v) {
-        fidelity = localDD->fidelity(edge, newEdge);
-    }
-
-    if (removeNodes) {
-        localDD->decRef(edge);
-        localDD->incRef(newEdge);
-        edge = newEdge;
-    }
-    return fidelity;
-}
-
 void StochasticNoiseSimulator::runStochSimulationForId(unsigned int                                stochRun,
                                                        int                                         n_qubits,
                                                        dd::Package::vEdge                          rootEdgePerfectRun,
                                                        std::vector<double>&                        recordedPropertiesStorage,
                                                        std::vector<std::tuple<long, std::string>>& recordedPropertiesList,
-                                                       unsigned long long                          local_seed) {
-    std::mt19937_64                        generator(local_seed);
+                                                       unsigned long long                          localSeed) {
+    std::mt19937_64                        generator(localSeed);
     std::uniform_real_distribution<dd::fp> dist(0.0, 1.0L);
 
     const unsigned long numberOfRuns = stochastic_runs / max_instances + (stochRun < stochastic_runs % max_instances ? 1 : 0);
     const int           approx_mod   = std::ceil(static_cast<double>(qc->getNops()) / (step_number + 1));
 
+    //        dd::NoiseOperationTable<dd::Package::mEdge> noiseOperationTable(getNumberOfQubits());
+
     //printf("Running %d times and using the dd at %p, using the cn object at %p\n", numberOfRuns, (void *) &localDD, (void *) &localDD->cn);
     for (unsigned long current_run = 0; current_run < numberOfRuns; current_run++) {
         const auto t1 = std::chrono::steady_clock::now();
 
         std::unique_ptr<dd::Package> localDD = std::make_unique<dd::Package>(getNumberOfQubits());
-        //dd::NoiseOperationTable<dd::Package::mEdge> noiseOperationTable(getNumberOfQubits());
-
-        dd::Package::mEdge identity_DD = localDD->makeIdent(n_qubits);
-        localDD->incRef(identity_DD);
-        dd::Package::vEdge local_root_edge = localDD->makeZeroState(n_qubits);
-        localDD->incRef(local_root_edge);
 
         std::map<unsigned int, bool> classic_values;
 
         unsigned int op_count     = 0;
         unsigned int approx_count = 0;
 
+        dd::Package::mEdge identity_DD = localDD->makeIdent(n_qubits);
+        localDD->incRef(identity_DD);
+        dd::Package::vEdge localRootEdge = localDD->makeZeroState(n_qubits);
+        localDD->incRef(localRootEdge);
+
         for (auto& op: *qc) {
             if (!op->isUnitary() && !(op->isClassicControlledOperation())) {
                 if (auto* nu_op = dynamic_cast<qc::NonUnitaryOperation*>(op.get())) {
@@ -326,8 +209,7 @@ void StochasticNoiseSimulator::runStochSimulationForId(unsigned int
                         assert(quantum.size() == classic.size()); // this should not happen do to check in Simulate
 
                         for (unsigned int i = 0; i < quantum.size(); ++i) {
-                            char result;
-                            local_root_edge = MeasureOneCollapsingConcurrent(quantum.at(i), localDD, local_root_edge, generator, &result);
+                            char result = localDD->measureOneCollapsing(localRootEdge, quantum.at(i), true, generator);
                             assert(result == '0' || result == '1');
                             classic_values[classic.at(i)] = (result == '1');
                         }
@@ -344,7 +226,6 @@ void StochasticNoiseSimulator::runStochSimulationForId(unsigned int
                 } else {
                     throw std::runtime_error("Dynamic cast to NonUnitaryOperation failed.");
                 }
-                localDD->garbageCollect(false);
             } else {
                 dd::Package::mEdge dd_op{};
                 qc::Targets        targets;
@@ -367,7 +248,7 @@ void StochasticNoiseSimulator::runStochSimulationForId(unsigned int
                     targets  = classic_op->getOperation()->getTargets();
                     controls = classic_op->getOperation()->getControls();
                     if (!execute_op) {
-                        // applyNoiseOperation(targets.front(), controls, identity_DD, (std::unique_ptr<dd::Package> &) localDD, local_root_edge, generator, dist, line, identity_DD);
+                        // applyNoiseOperation(targets.front(), controls, identity_DD, (std::unique_ptr<dd::Package> &) localDD, localRootEdge, generator, dist, line, identity_DD);
                         continue;
                     }
                 } else {
@@ -376,16 +257,16 @@ void StochasticNoiseSimulator::runStochSimulationForId(unsigned int
                     controls = op->getControls();
                 }
 
-                applyNoiseOperation(targets, controls, dd_op, localDD, local_root_edge, generator, dist, identity_DD);
+                applyNoiseOperation(targets, controls, dd_op, localDD, localRootEdge, generator, dist, identity_DD);
                 if (step_fidelity < 1 && (op_count + 1) % approx_mod == 0) {
-                    ApproximateEdgeByFidelity(localDD, local_root_edge, step_fidelity, false, true);
+                    ApproximateByFidelity(localDD, localRootEdge, step_fidelity, false, true);
                     approx_count++;
                 }
-                //localDD->garbageCollect(false);
             }
+            localDD->garbageCollect();
             op_count++;
         }
-        localDD->decRef(local_root_edge);
+        localDD->decRef(localRootEdge);
         const auto t2 = std::chrono::steady_clock::now();
 
         for (unsigned long i = 0; i < recordedPropertiesStorage.size(); i++) {
@@ -394,11 +275,11 @@ void StochasticNoiseSimulator::runStochSimulationForId(unsigned int
             } else if (std::get<0>(recordedPropertiesList[i]) == -2) {
                 recordedPropertiesStorage[i] += approx_count;
             } else if (std::get<0>(recordedPropertiesList[i]) == -1) {
-                recordedPropertiesStorage[i] += localDD->fidelity(local_root_edge, rootEdgePerfectRun);
+                recordedPropertiesStorage[i] += localDD->fidelity(localRootEdge, rootEdgePerfectRun);
             } else {
                 // extract amplitude for state
                 const auto basisVector = std::get<1>(recordedPropertiesList[i]);
-                auto       amplitude   = localDD->getValueByPath(local_root_edge, basisVector);
+                auto       amplitude   = localDD->getValueByPath(localRootEdge, basisVector);
                 auto       prob        = amplitude.r * amplitude.r + amplitude.i * amplitude.i;
                 recordedPropertiesStorage[i] += prob;
             }
@@ -410,41 +291,33 @@ void StochasticNoiseSimulator::applyNoiseOperation(const qc::Targets&
                                                    const dd::Controls&                     control_qubits,
                                                    dd::Package::mEdge                      dd_op,
                                                    std::unique_ptr<dd::Package>&           localDD,
-                                                   dd::Package::vEdge&                     local_root_edge,
+                                                   dd::Package::vEdge&                     localRootEdge,
                                                    std::mt19937_64&                        generator,
                                                    std::uniform_real_distribution<dd::fp>& dist,
                                                    dd::Package::mEdge                      identityDD) {
-    // TODO: Is this a meaningful way to handle multi-target operations?
-    for (auto& target: targets) {
+    std::vector usedQubits = targets;
+    for (auto control: control_qubits) {
+        usedQubits.push_back(control.qubit);
+    }
+
+    for (auto& target: usedQubits) {
         auto operation = generateNoiseOperation(false, target, generator, dist, dd_op, localDD);
-        auto tmp       = localDD->multiply(operation, local_root_edge);
+        auto tmp       = localDD->multiply(operation, localRootEdge);
 
         if (dd::ComplexNumbers::mag2(tmp.w) < dist(generator)) {
             operation = generateNoiseOperation(true, target, generator, dist, dd_op, localDD);
-            tmp       = localDD->multiply(operation, local_root_edge);
+            tmp       = localDD->multiply(operation, localRootEdge);
         }
 
         if (tmp.w != dd::Complex::one) {
             tmp.w = dd::Complex::one;
         }
         localDD->incRef(tmp);
-        localDD->decRef(local_root_edge);
-        local_root_edge = tmp;
-    }
-    // Apply noise to control qubits
-    for (auto control: control_qubits) {
-        auto operation = generateNoiseOperation(false, control.qubit, generator, dist, identityDD, localDD);
-        auto tmp       = localDD->multiply(operation, local_root_edge);
-        if (dd::ComplexNumbers::mag2(tmp.w) < dist(generator)) {
-            operation = generateNoiseOperation(true, control.qubit, generator, dist, identityDD, localDD);
-            tmp       = localDD->multiply(operation, local_root_edge);
-        }
-        if (tmp.w != dd::Complex::one) {
-            tmp.w = dd::Complex::one;
-        }
-        localDD->incRef(tmp);
-        localDD->decRef(local_root_edge);
-        local_root_edge = tmp;
+        localDD->decRef(localRootEdge);
+        localRootEdge = tmp;
+
+        // I only need to apply the operations once
+        dd_op = identityDD;
     }
 }
 
@@ -507,123 +380,6 @@ dd::Package::mEdge StochasticNoiseSimulator::generateNoiseOperation(bool
     return dd_operation;
 }
 
-dd::Package::vEdge StochasticNoiseSimulator::MeasureOneCollapsingConcurrent(unsigned short                      index,
-                                                                            const std::unique_ptr<dd::Package>& localDD,
-                                                                            dd::Package::vEdge                  local_root_edge,
-                                                                            std::mt19937_64&                    generator,
-                                                                            char*                               result,
-                                                                            bool                                assume_probability_normalization) {
-    //todo merge the function with MeasureOneCollapsing
-    std::map<dd::Package::vNode*, dd::fp> probsMone;
-    std::set<dd::Package::vNode*>         visited_nodes2;
-    std::queue<dd::Package::vNode*>       q;
-
-    probsMone[local_root_edge.p] = dd::ComplexNumbers::mag2(local_root_edge.w);
-    visited_nodes2.insert(local_root_edge.p);
-    q.push(local_root_edge.p);
-
-    while (q.front()->v != index) {
-        dd::Package::vNode* ptr = q.front();
-        q.pop();
-        double prob = probsMone[ptr];
-
-        if (!ptr->e.at(0).w.approximatelyZero()) {
-            const dd::fp tmp1 = prob * dd::ComplexNumbers::mag2(ptr->e.at(0).w);
-
-            if (visited_nodes2.find(ptr->e.at(0).p) != visited_nodes2.end()) {
-                probsMone[ptr->e.at(0).p] = probsMone[ptr->e.at(0).p] + tmp1;
-            } else {
-                probsMone[ptr->e.at(0).p] = tmp1;
-                visited_nodes2.insert(ptr->e.at(0).p);
-                q.push(ptr->e.at(0).p);
-            }
-        }
-
-        if (!ptr->e.at(1).w.approximatelyZero()) {
-            const dd::fp tmp1 = prob * dd::ComplexNumbers::mag2(ptr->e.at(1).w);
-
-            if (visited_nodes2.find(ptr->e.at(1).p) != visited_nodes2.end()) {
-                probsMone[ptr->e.at(1).p] = probsMone[ptr->e.at(1).p] + tmp1;
-            } else {
-                probsMone[ptr->e.at(1).p] = tmp1;
-                visited_nodes2.insert(ptr->e.at(1).p);
-                q.push(ptr->e.at(1).p);
-            }
-        }
-    }
-
-    dd::fp pzero{0}, pone{0};
-
-    if (assume_probability_normalization) {
-        while (!q.empty()) {
-            dd::Package::vNode* ptr = q.front();
-            q.pop();
-
-            if (!ptr->e.at(0).w.approximatelyZero()) {
-                pzero += probsMone[ptr] * dd::ComplexNumbers::mag2(ptr->e.at(0).w);
-            }
-
-            if (!ptr->e.at(1).w.approximatelyZero()) {
-                pone += probsMone[ptr] * dd::ComplexNumbers::mag2(ptr->e.at(1).w);
-            }
-        }
-    } else {
-        std::unordered_map<dd::Package::vNode*, double> probs;
-        assign_probs(root_edge, probs);
-
-        while (!q.empty()) {
-            dd::Package::vNode* ptr = q.front();
-            q.pop();
-
-            if (!ptr->e.at(0).w.approximatelyZero()) {
-                pzero += probsMone[ptr] * probs[ptr->e.at(0).p] * dd::ComplexNumbers::mag2(ptr->e.at(0).w);
-            }
-
-            if (!ptr->e.at(1).w.approximatelyZero()) {
-                pone += probsMone[ptr] * probs[ptr->e.at(1).p] * dd::ComplexNumbers::mag2(ptr->e.at(1).w);
-            }
-        }
-    }
-
-    if (std::abs(pzero + pone - 1) > epsilon) {
-        throw std::runtime_error("Numerical instability occurred during measurement: |alpha|^2 + |beta|^2 = " + std::to_string(pzero) + " + " + std::to_string(pone) + " = " + std::to_string(pzero + pone) + ", but should be 1!");
-    }
-
-    const dd::fp sum = pzero + pone;
-
-    //std::pair<fp, fp> probs = std::make_pair(pzero, pone);
-    dd::GateMatrix measure_m{
-            dd::complex_zero, dd::complex_zero,
-            dd::complex_zero, dd::complex_zero};
-
-    std::uniform_real_distribution<dd::fp> dist(0.0, 1.0L);
-
-    dd::fp n = dist(generator);
-    dd::fp norm_factor;
-
-    if (n < pzero / sum) {
-        measure_m[0] = dd::complex_one;
-        norm_factor  = pzero;
-        *result      = '0';
-    } else {
-        measure_m[3] = dd::complex_one;
-        norm_factor  = pone;
-        *result      = '1';
-    }
-    dd::Edge m_gate = localDD->makeGateDD(measure_m, getNumberOfQubits(), index);
-
-    dd::Edge e = localDD->multiply(m_gate, local_root_edge);
-
-    dd::Complex c = localDD->cn.getTemporary(std::sqrt(1 / norm_factor), 0);
-    dd::ComplexNumbers::mul(c, e.w, c);
-    e.w = localDD->cn.lookup(c);
-    localDD->incRef(e);
-    localDD->decRef(local_root_edge);
-    local_root_edge = e;
-
-    return local_root_edge;
-}
-
 dd::NoiseOperationKind StochasticNoiseSimulator::ReturnNoiseOperation(char i, double prob) const {
     //    if (forcedResult.size() > currentResult) {
     //        auto tmp = forcedResult[currentResult];
@@ -641,7 +397,7 @@ dd::NoiseOperationKind StochasticNoiseSimulator::ReturnNoiseOperation(char i, do
             } else if (prob < noise_probability / 2) {
                 //                printf("Y");
                 return dd::Y;
-            } else { //if (prob < 3 * noise_probability / 4) {
+            } else { //if (prob < 3 * noiseProbability / 4) {
                      //                printf("Z");
                 return dd::Z;
             }
@@ -725,3 +481,19 @@ std::string StochasticNoiseSimulator::intToString(long target_number) const {
     }
     return path;
 }
+void StochasticNoiseSimulator::setMeasuredQubitToZero(signed char& at, dd::Package::vEdge& e, std::unique_ptr<dd::Package>& localDD) {
+    auto f = dd::Package::mEdge::one;
+
+    for (std::size_t p = 0; p < getNumberOfQubits(); p++) {
+        if (static_cast<dd::Qubit>(p) == at) {
+            f = localDD->makeDDNode(static_cast<dd::Qubit>(p), std::array{f, f, dd::Package::mEdge::zero, dd::Package::mEdge::zero});
+            //                f = dd->makeDDNode(static_cast<dd::Qubit>(p), std::array{f, dd::Package::mEdge::zero, dd::Package::mEdge::zero, dd::Package::mEdge::zero});
+        } else {
+            f = localDD->makeDDNode(static_cast<dd::Qubit>(p), std::array{f, dd::Package::mEdge::zero, dd::Package::mEdge::zero, f});
+        }
+    }
+    auto tmp = localDD->multiply(f, e);
+    localDD->incRef(tmp);
+    localDD->decRef(e);
+    e = tmp;
+}
